/** 
 * \file MagicFlower.h
 * \brief Header file containing CMagicFlower class encapsulating magic flower functionality.
 * 
 * \author Petar Bajic, MPE (C) All Rights Reserved, Homepage: www.mystic-peanut.com
 * \date January, 16 2007.
 */

#include "d3dx9.h"
#include "ModelStatic.h"
#include "Particles.h"

/**
 * \brief Magic flowers add special bonus to turtle gun.
 *
 * Magic flowers are static models with particle flame above them.
 * \author Petar Bajic \date January, 16 2007.
 */
class CMagicFlower
{
public:
	CMagicFlower();
	~CMagicFlower();
	bool Load(HWND hWnd, LPDIRECT3DDEVICE9 pd3dDevice, WCHAR* strFilePath, WCHAR* strFileName, D3DCOLOR color);
	void SetPosition(D3DXVECTOR3 position);
	void OnLostDevice();
	void OnResetDevice(LPDIRECT3DDEVICE9 pd3dDevice);
	void OnRender(LPDIRECT3DDEVICE9 pd3dDevice);
	void OnFrameMove(LPDIRECT3DDEVICE9 pd3dDevice, float elapsed);

	bool m_Alive;
	CFlame* m_ParticleFlame;
	CModelStatic* m_Model;
	float m_TimeOn;
	float m_TimeOff;
	D3DXVECTOR3 m_Position;
	float m_Rotation;
	D3DCOLOR m_Color;
};